package Tabu;

import Utils.Action;
import VoteReveal.RevealOrder;
import VoteReveal.RevealProblem;
import VoteReveal.VRPDataReader;

import java.util.ArrayList;
import java.util.List;

public class SingleObjectiveTabuSearch {

    // args: filename, func, tenure, neighbourhoodfunc, numNonImprove
    public static void main(String[] args) {
        assert args.length > 5;
        String filename = args[0];
        char func = args[1].toCharArray()[0];
        char[] funcs = func == 'A' ?
                new char[]{'B','L','P','Q','T','U'} :
                new char[]{func};
        int tenure = Integer.parseInt(args[2]);
        char neighbourhood = args[3].toCharArray()[0];
        int numNonImprove = Integer.parseInt(args[4]);

        RevealProblem problem = VRPDataReader.readData(filename);

        for (char f : funcs) {
            for (int i = 0; i < 50; i++) {

                // get initial solution
                RevealOrder X = problem.getRandomSolution();
                RevealOrder Best = X;

                // initialise tabu list
                TabuList L = new TabuList(tenure);

                int solutionsSinceImprovement = 0;
                double startTime = System.currentTimeMillis();
                int count = 0;
                while (solutionsSinceImprovement < numNonImprove) {
                    count++;

                    List<Action> legalActions = new ArrayList<>();
                    List<Action> actions = generateActions(neighbourhood, problem.voters.length);
                    for (Action a : actions) if (!L.contains(a)) legalActions.add(a);

                    RevealOrder[] Neighbours = X.getNeighbours(legalActions, func);
                    X = Neighbours[0].copy();
                    L.add(X.getAction());

                    if (X.getCost(f) > Best.getCost(f)) {
                        Best = X.copy();
                        solutionsSinceImprovement = 0;
                    } else {
                        solutionsSinceImprovement++;
                    }
                }
                double stopTime = System.currentTimeMillis();

                System.out.println(Best);
                System.out.printf("Runtime: %fs%n", (stopTime - startTime) / 1000);
                System.out.println(count);

                Best.write(f, "TS_"+tenure+"_"+neighbourhood+"_"+numNonImprove);
            }
        }
    }

    private static List<Action> generateActions(char neighbourhood, int v) {
        return switch(neighbourhood) {
            case 'R' -> {
                List<Action> a = new ArrayList<>();
                for (int i = 0; i < v-1; i++) {
                    a.add(new Action(i, i+1));
                }
                yield a;
            }
            case 'F' -> {
                List<Action> a = new ArrayList<>();
                for (int i = 0; i < v-1; i++) {
                    for (int j = i+1; j < v; j++) {
                        a.add(new Action(i,j));
                    }
                }
                yield a;
            }
            default -> throw new IllegalStateException("Unexpected input for neighbourhood: " + neighbourhood);
        };
    }
}
