package Visualisation;

import Utils.IndexedValue;
import VoteReveal.RevealOrder;
import VoteReveal.RevealProblem;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.CategoryPlot;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.renderer.category.CategoryItemRenderer;
import org.jfree.data.category.CategoryDataset;
import org.jfree.data.category.DefaultCategoryDataset;

import java.awt.*;
import java.util.Arrays;

public class BarChart {
    private final String chartTitle;
    private final RevealOrder solution;

    public BarChart(String title, RevealOrder solution) {
        chartTitle = title;
        this.solution = solution;
    }

    public JFreeChart generateGraph(int r, boolean sort) {
        JFreeChart chart = ChartFactory.createBarChart(
                chartTitle,
                "Candidates",
                "Points",
                getDataSet(r, sort),
                PlotOrientation.VERTICAL,
                true, true, false
        );
        final CategoryPlot plot = (CategoryPlot) chart.getPlot();
        plot.setBackgroundPaint(new Color(240, 240, 240));
        plot.setRenderer(new CustomBarRenderer());
        return chart;
    }

    private CategoryDataset getDataSet(int r, boolean sort) {
        final DefaultCategoryDataset dataset = new DefaultCategoryDataset();
        int[][] cumulativeVals = solution.getCumulativeVals();

        // Create set of indexed values
        IndexedValue[] row = new IndexedValue[cumulativeVals[r].length];
        for (int i = 0; i < row.length; i++)
            row[i] = new IndexedValue(i, cumulativeVals[r][i]);

        if (sort) Arrays.sort(row);
        SingleSolutionVisualiser.setData(row);

        for (int i = 0; i < row.length; i++) {
            int value = row[i].value;
            String label = solution.getCandidate(row[i].index);
            dataset.addValue(value, "", label);
        }

        return dataset;
    }
}
