package Visualisation;

import VoteReveal.RevealOrder;
import VoteReveal.RevealProblem;
import VoteReveal.VRPDataReader;

import org.jfree.chart.ChartPanel;

import javax.swing.*;
import java.io.File;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.List;
import java.util.Scanner;

public class MultiSolutionVisualiser {


    public static void main(String[] args) {
        assert args.length > 1;
        String solutionFile = args[0];

        // Get funcs from filename
        int underscore = solutionFile.lastIndexOf("_")+1;
        int dot = solutionFile.lastIndexOf(".");

        char[] funcs = solutionFile.substring(underscore, dot).toCharArray();
        String instanceFile = solutionFile.substring(0, underscore-1).concat(".txt");
        RevealProblem problem = VRPDataReader.readData(instanceFile);
        assert problem != null;

        List<RevealOrder> set = new ArrayList<>();

        try {
            File f = new File(solutionFile);
            Scanner sc = new Scanner(f);

            // Read solutions
            while (sc.hasNextLine()) {
                Scanner rowScanner = new Scanner(sc.nextLine());
                rowScanner.useDelimiter(",");
                for (int i = 0; i < funcs.length; i++) {
                    rowScanner.next();
                    rowScanner.next();
                } // Discard function values
                rowScanner.next(); // Discard label

                // Get reveal order
                List<Integer> order = new ArrayList<>();
                while (rowScanner.hasNext())
                    order.add(Integer.parseInt(rowScanner.next()));

                int[] o = new int[order.size()];
                for (int i = 0; i < order.size(); i++)
                    o[i] = order.get(i);

                set.add(new RevealOrder(o, problem));
            }
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }

        // Convert set to array
        RevealOrder[] solutionArray = set.toArray(new RevealOrder[0]);

        // Create Swing GUI window
        final JFrame frame = new JFrame("Solution Set Visualiser");
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame.setSize(500, 300);

        // Add spider chart
        ChartPanel chart = new ChartPanel(null);
        frame.add(chart);
        SpiderChart spider = new SpiderChart(problem.title, solutionArray, funcs);
        chart.setChart(spider.generateChart());
        frame.getContentPane().add(chart);
        frame.setVisible(true);
    }
}
