package Visualisation;

import Utils.IndexedValue;
import VoteReveal.RevealOrder;
import VoteReveal.RevealProblem;
import VoteReveal.VRPDataReader;

import org.jfree.chart.ChartPanel;

import javax.swing.*;
import java.awt.*;
import java.util.ArrayList;
import java.util.List;

public class SingleSolutionVisualiser {
    private static int r = 0;
    private static int max;
    private static BarChart barChart;
    private static ChartPanel chart;

    private static SolutionAnalyser analyser;

    // GUI Settings
    private static JCheckBox leaders;
    private static JCheckBox ties;
    private static JCheckBox poles;
    private static JCheckBox overtakes;
    private static JCheckBox sort;

    private static JLabel revealNumber;

    // Display order of bars
    private static IndexedValue[] barOrder;

    public static void main(String[] args) {
        assert args.length > 1;
        String instanceFile = args[0];

        // Read list of integers
        List<Integer> orderList = new ArrayList<>();
        for (String o : args[1].split(","))
            orderList.add(Integer.parseInt(o));

        RevealProblem problem = VRPDataReader.readData(instanceFile);

        int numVoters = problem.voters.length;
        max = numVoters-1;
        assert orderList.size() == numVoters;
        int[] order = new int[numVoters];
        int i = 0;
        for (Integer j : orderList) {
            order[i] = j;
            i++;
        }

        RevealOrder solution = new RevealOrder(order, problem);
        System.out.println(solution);
        analyser = new SolutionAnalyser(solution);

        // Create Swing GUI window
        final JFrame frame = new JFrame("Solution Visualiser");
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame.setSize(1280, 720);

        // Add bar chart
        chart = new ChartPanel(null);
        frame.add(chart);
        barChart = new BarChart(problem.title, solution);
        chart.setChart(barChart.generateGraph(r, false));
        frame.getContentPane().add(chart);
        frame.setVisible(true);

        // Button panel
        JPanel panel = new JPanel();
        revealNumber = new JLabel(""+(r+1));
        panel.add(revealNumber);

        JButton prev = new JButton("prev");
        prev.addActionListener(e -> prev());
        panel.add(prev);

        JButton next = new JButton("next");
        next.addActionListener(e -> next());
        panel.add(next);

        leaders   = new JCheckBox("Highlight leader");
        poles     = new JCheckBox("Highlight previous leaders");
        ties      = new JCheckBox("Highlight ties");
        overtakes = new JCheckBox("Highlight position changes");
             sort = new JCheckBox("Sort");
          leaders.addActionListener(e -> update());
            poles.addActionListener(e -> update());
             ties.addActionListener(e -> update());
        overtakes.addActionListener(e -> update());
             sort.addActionListener(e -> update());
        panel.add(leaders);
        panel.add(poles);
        panel.add(ties);
        panel.add(overtakes);
        panel.add(sort);

        frame.getContentPane().add(BorderLayout.SOUTH, panel);
    }

    private static void prev() {
        r = Math.max(r-1, 0);
        update();
    }

    private static void next() {
        r = Math.min(r+1, max);
        update();
    }

    private static void update() {
        chart.setChart(barChart.generateGraph(r, sort.isSelected()));
        revealNumber.setText(""+(r+1));
    }

    public static Paint getColour(int i) {
        int j = barOrder[i].index;
        Color color = new Color(128, 128, 128);

        try {
            if (overtakes.isSelected()) {
                int dR = analyser.rankingChange(j, r);

                if (dR > 1) color = new Color(61, 193, 0);
                else if (dR == 1) color = new Color(146, 201, 122);
                else if (dR < -1) color = new Color(255, 0, 0);
                else if (dR == -1) color = new Color(139, 69, 69);
            }

            if (analyser.isLeader(j, r) && leaders.isSelected())
                color = new Color(255, 184, 0);

            else if (analyser.isTied(j, r) && ties.isSelected())
                color = new Color(38, 222, 232);

            else if (analyser.isPreviousPole(j, r) && poles.isSelected())
                color = new Color(199, 172, 101);

            return color;
        }
        catch (NullPointerException e) {
            return color;
        }
    }

    public static void setData(IndexedValue[] row) {
        barOrder = row;
    }
}
