package Visualisation;

import Utils.IndexedValue;
import VoteReveal.RevealOrder;

import java.util.Arrays;

class SolutionAnalyser {
    private final int[][] cumulativeVals;
    private final int[] maxVals;
    private final int[][] rankings;
    private final boolean[][] ties;
    private final boolean[][] previousPoles;

    public SolutionAnalyser(RevealOrder order) {
        cumulativeVals = order.getCumulativeVals();

        int v = cumulativeVals.length;
        int c = cumulativeVals[0].length;

        maxVals = new int[v];
        rankings = new int[v][c];
        ties = new boolean[v][c];
        previousPoles = new boolean[v][c];

        IndexedValue[][] s = new IndexedValue[v][c];
        for (int i = 0; i < v; i++) {

            for (int j = 0; j < cumulativeVals[i].length; j++) {
                // Propagate previous winners forward
                if (i>0 && previousPoles[i-1][j]) previousPoles[i][j] = true;

                // Use IndexedValues to sort candidates by point value while still retaining
                // their index (candidate number)
                s[i][j] = new IndexedValue(j, cumulativeVals[i][j]);
            }

            int n = s[i].length;

            Arrays.sort(s[i]);
            maxVals[i] = s[i][0].value; // Store the max points for this round

            // Mark the winner(s) as a previous winner in the next round.
            if (i < v-1) {
                int a = 0;
                while (s[i][a].value == maxVals[i] && a < v-1) {
                    previousPoles[i+1][s[i][a].index] = true;
                    a++;
                }
            }

            // Ties and rankings
            int rank = 0;
            int rankDisp = 1;
            for (int k = 0; k < n; k++) {
                // Mark ties in boolean matrix
                if (k > 0 && s[i][k].value == s[i][k-1].value) { // Compare current to previous
                    ties[i][s[i][k].index] = true;
                    rankDisp++;
                }
                else if (k < n-1 && s[i][k].value == s[i][k+1].value) { // Compare current to next
                    ties[i][s[i][k].index] = true;
                    rank += rankDisp;
                    rankDisp = 1;
                }
                else {
                    rank += rankDisp;
                    rankDisp = 1;
                }

                rankings[i][s[i][k].index] = rank; // Store this candidate's rank (accounting for ties)
            }
        }
    }

    public boolean isLeader(int candidate, int reveal) {
        return cumulativeVals[reveal][candidate] == maxVals[reveal];
    }

    public int rankingChange(int candidate, int reveal) {
        if (reveal > 0)
            return rankings[reveal-1][candidate] - rankings[reveal][candidate];
        else
            return 0;
    }

    public boolean isTied(int candidate, int reveal) {
        return ties[reveal][candidate];
    }

    public boolean isPreviousPole(int candidate, int reveal) {
        return previousPoles[reveal][candidate];
    }
}
