package Visualisation;

import VoteReveal.RevealOrder;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.SpiderWebPlot;
import org.jfree.data.category.DefaultCategoryDataset;

// @see https://stackoverflow.com/a/32885067/230513
public class SpiderChart {
    private final String chartTitle;
    private final RevealOrder[] solutions;
    private final char[] funcs;

    public SpiderChart(String chartTitle, RevealOrder[] solutions, char[] funcs) {
        this.chartTitle = chartTitle;
        this.solutions = solutions;
        this.funcs = funcs;
    }

    public JFreeChart generateChart() {
        // Generate dataset
        DefaultCategoryDataset dataset = new DefaultCategoryDataset();

        float[][] data = new float[funcs.length][solutions.length];
        for (int f = 0; f < funcs.length; f++) {
            float min = Integer.MAX_VALUE;
            float max = Integer.MIN_VALUE;
            for (int sol = 0; sol < solutions.length; sol++) {
                float val = solutions[sol].getCost(funcs[f]);
                min = Math.min(min, val);
                max = Math.max(max, val);
                data[f][sol] = val;
            }
            for (int sol = 0; sol < solutions.length; sol++) {
                data[f][sol] = (data[f][sol]-min)/(max-min);
            }
        }

        for (int sol = 0; sol < solutions.length; sol++) {
            String rowKey = "Solution " + (sol+1);
//            String rowKey = switch(sol) {
//                case 0 -> "UNC(C)";
//                case 1 -> "LEAD";
//                case 2 -> "HOT(U)";
//                case 3 -> "HOT";
//                case 4 -> "UNC(T)";
//                case 5 -> "UNC";
//                default -> throw new IllegalStateException("Unexpected value: " + sol);
//            };
            for (int func = 0; func < funcs.length; func++) {
                String colKey = switch(funcs[func]) {
                    case 'B' -> "Candidate-Biased Uncertainty";
                    case 'L' -> "Lead";
                    case 'P' -> "Unique Hotseat";
                    case 'Q' -> "Hotseat";
                    case 'T' -> "Time-Biased Uncertainty";
                    case 'U' -> "Uncertainty";
                    default -> throw new IllegalStateException("Unexpected value: " + funcs[func]);
                };
                float val = Float.isNaN(data[func][sol]) ? 0.5f : data[func][sol]+0.5f;
                dataset.addValue(val, rowKey, colKey);
            }
        }

        // Create and return chart
        SpiderWebPlot plot = new SpiderWebPlot(dataset);
        return new JFreeChart(chartTitle, plot);
    }
}

